@@ -41,6 +41,7 @@ from watcher.api.controllers.v1 import utils as api_utils
 from watcher.common import exception
 from watcher.common import policy
 from watcher.common import utils as common_utils
+from watcher.decision_engine import rpcapi
 from watcher import objects
 
 
@@ -172,17 +173,6 @@ class StrategyCollection(collection.Collection):
         strategy_collection = StrategyCollection()
         strategy_collection.strategies = [
             Strategy.convert_with_links(g, expand) for g in strategies]
-
-        if 'sort_key' in kwargs:
-            reverse = False
-            if kwargs['sort_key'] == 'strategy':
-                if 'sort_dir' in kwargs:
-                    reverse = True if kwargs['sort_dir'] == 'desc' else False
-                strategy_collection.strategies = sorted(
-                    strategy_collection.strategies,
-                    key=lambda strategy: strategy.uuid,
-                    reverse=reverse)
-
         strategy_collection.next = strategy_collection.get_next(
             limit, url=url, **kwargs)
         return strategy_collection
@@ -205,32 +195,44 @@ class StrategiesController(rest.RestController):
 
     _custom_actions = {
         'detail': ['GET'],
+        'state': ['GET'],
     }
 
     def _get_strategies_collection(self, filters, marker, limit, sort_key,
                                    sort_dir, expand=False, resource_url=None):
+        additional_fields = ["goal_uuid", "goal_name"]
+
+        api_utils.validate_sort_key(
+            sort_key, list(objects.Strategy.fields) + additional_fields)
         api_utils.validate_search_filters(
-            filters, list(objects.strategy.Strategy.fields) +
-            ["goal_uuid", "goal_name"])
+            filters, list(objects.Strategy.fields) + additional_fields)
         limit = api_utils.validate_limit(limit)
         api_utils.validate_sort_dir(sort_dir)
 
-        sort_db_key = (sort_key if sort_key in objects.Strategy.fields
-                       else None)
-
         marker_obj = None
         if marker:
             marker_obj = objects.Strategy.get_by_uuid(
                 pecan.request.context, marker)
 
+        need_api_sort = api_utils.check_need_api_sort(sort_key,
+                                                      additional_fields)
+        sort_db_key = (sort_key if not need_api_sort
+                       else None)
+
         strategies = objects.Strategy.list(
             pecan.request.context, limit, marker_obj, filters=filters,
             sort_key=sort_db_key, sort_dir=sort_dir)
 
-        return StrategyCollection.convert_with_links(
+        strategies_collection = StrategyCollection.convert_with_links(
             strategies, limit, url=resource_url, expand=expand,
             sort_key=sort_key, sort_dir=sort_dir)
 
+        if need_api_sort:
+            api_utils.make_api_sort(strategies_collection.strategies,
+                                    sort_key, sort_dir)
+
+        return strategies_collection
+
     @wsme_pecan.wsexpose(StrategyCollection, wtypes.text, wtypes.text,
                          int, wtypes.text, wtypes.text)
     def get_all(self, goal=None, marker=None, limit=None,
@@ -288,6 +290,26 @@ class StrategiesController(rest.RestController):
         return self._get_strategies_collection(
             filters, marker, limit, sort_key, sort_dir, expand, resource_url)
 
+    @wsme_pecan.wsexpose(wtypes.text, wtypes.text)
+    def state(self, strategy):
+        """Retrieve a inforamation about strategy requirements.
+
+        :param strategy: name of the strategy.
+        """
+        context = pecan.request.context
+        policy.enforce(context, 'strategy:state', action='strategy:state')
+        parents = pecan.request.path.split('/')[:-1]
+        if parents[-2] != "strategies":
+            raise exception.HTTPNotFound
+        rpc_strategy = api_utils.get_resource('Strategy', strategy)
+        de_client = rpcapi.DecisionEngineAPI()
+        strategy_state = de_client.get_strategy_info(context,
+                                                     rpc_strategy.name)
+        strategy_state.extend([{
+            'type': 'Name', 'state': rpc_strategy.name,
+            'mandatory': '', 'comment': ''}])
+        return strategy_state
+
     @wsme_pecan.wsexpose(Strategy, wtypes.text)
     def get_one(self, strategy):
         """Retrieve information about the given strategy.
