@@ -19,21 +19,32 @@
 
 import mock
 
+from watcher.common import clients
 from watcher.common import utils
 from watcher.decision_engine.model import model_root
 from watcher.decision_engine.strategy import strategies
 from watcher.tests import base
 from watcher.tests.decision_engine.model import ceilometer_metrics
 from watcher.tests.decision_engine.model import faker_cluster_state
+from watcher.tests.decision_engine.model import gnocchi_metrics
 
 
 class TestWorkloadStabilization(base.TestCase):
 
+    scenarios = [
+        ("Ceilometer",
+         {"datasource": "ceilometer",
+          "fake_datasource_cls": ceilometer_metrics.FakeCeilometerMetrics}),
+        ("Gnocchi",
+         {"datasource": "gnocchi",
+          "fake_datasource_cls": gnocchi_metrics.FakeGnocchiMetrics}),
+    ]
+
     def setUp(self):
         super(TestWorkloadStabilization, self).setUp()
 
         # fake metrics
-        self.fake_metrics = ceilometer_metrics.FakeCeilometerMetrics()
+        self.fake_metrics = self.fake_datasource_cls()
 
         # fake cluster
         self.fake_cluster = faker_cluster_state.FakerModelCollector()
@@ -45,17 +56,22 @@ class TestWorkloadStabilization(base.TestCase):
             'Node_3': {'cpu_util': 0.05, 'memory.resident': 8, 'vcpus': 40},
             'Node_4': {'cpu_util': 0.05, 'memory.resident': 4, 'vcpus': 40}}
 
+        p_osc = mock.patch.object(
+            clients, "OpenStackClients")
+        self.m_osc = p_osc.start()
+        self.addCleanup(p_osc.stop)
+
         p_model = mock.patch.object(
             strategies.WorkloadStabilization, "compute_model",
             new_callable=mock.PropertyMock)
         self.m_model = p_model.start()
         self.addCleanup(p_model.stop)
 
-        p_ceilometer = mock.patch.object(
-            strategies.WorkloadStabilization, "ceilometer",
+        p_datasource = mock.patch.object(
+            strategies.WorkloadStabilization, "datasource_backend",
             new_callable=mock.PropertyMock)
-        self.m_ceilometer = p_ceilometer.start()
-        self.addCleanup(p_ceilometer.stop)
+        self.m_datasource = p_datasource.start()
+        self.addCleanup(p_datasource.stop)
 
         p_audit_scope = mock.patch.object(
             strategies.WorkloadStabilization, "audit_scope",
@@ -65,10 +81,12 @@ class TestWorkloadStabilization(base.TestCase):
         self.addCleanup(p_audit_scope.stop)
 
         self.m_model.return_value = model_root.ModelRoot()
-        self.m_ceilometer.return_value = mock.Mock(
-            statistic_aggregation=self.fake_metrics.mock_get_statistics)
         self.m_audit_scope.return_value = mock.Mock()
-        self.strategy = strategies.WorkloadStabilization(config=mock.Mock())
+        self.m_datasource.return_value = mock.Mock(
+            statistic_aggregation=self.fake_metrics.mock_get_statistics)
+
+        self.strategy = strategies.WorkloadStabilization(
+            config=mock.Mock(datasource=self.datasource))
         self.strategy.input_parameters = utils.Struct()
         self.strategy.input_parameters.update(
             {'metrics': ["cpu_util", "memory.resident"],
@@ -80,7 +98,8 @@ class TestWorkloadStabilization(base.TestCase):
                   "memory.resident": "hardware.memory.used"},
              'host_choice': 'retry',
              'retry_count': 1,
-             'periods': {"instance": 720, "node": 600}})
+             'periods': {"instance": 720, "node": 600},
+             'aggregation_method': {"instance": "mean", "node": "mean"}})
         self.strategy.metrics = ["cpu_util", "memory.resident"]
         self.strategy.thresholds = {"cpu_util": 0.2, "memory.resident": 0.2}
         self.strategy.weights = {"cpu_util_weight": 1.0,
@@ -91,6 +110,7 @@ class TestWorkloadStabilization(base.TestCase):
         self.strategy.host_choice = 'retry'
         self.strategy.retry_count = 1
         self.strategy.periods = {"instance": 720, "node": 600}
+        self.strategy.aggregation_method = {"instance": "mean", "node": "mean"}
 
     def test_get_instance_load(self):
         model = self.fake_cluster.generate_scenario_1()
@@ -102,24 +122,11 @@ class TestWorkloadStabilization(base.TestCase):
         self.assertEqual(
             instance_0_dict, self.strategy.get_instance_load(instance0))
 
-    def test_periods(self):
-        model = self.fake_cluster.generate_scenario_1()
+    def test_get_instance_load_with_no_metrics(self):
+        model = self.fake_cluster.generate_scenario_1_with_1_node_unavailable()
         self.m_model.return_value = model
-        p_ceilometer = mock.patch.object(
-            strategies.WorkloadStabilization, "ceilometer")
-        m_ceilometer = p_ceilometer.start()
-        self.addCleanup(p_ceilometer.stop)
-        m_ceilometer.return_value = mock.Mock(
-            statistic_aggregation=self.fake_metrics.mock_get_statistics)
-        instance0 = model.get_instance_by_uuid("INSTANCE_0")
-        self.strategy.get_instance_load(instance0)
-        m_ceilometer.statistic_aggregation.assert_called_with(
-            aggregate='min', meter_name='memory.resident',
-            period=720, resource_id=instance0.uuid)
-        self.strategy.get_hosts_load()
-        m_ceilometer.statistic_aggregation.assert_called_with(
-            aggregate='avg', meter_name='hardware.memory.used',
-            period=600, resource_id=mock.ANY)
+        lost_instance = model.get_instance_by_uuid("LOST_INSTANCE")
+        self.assertIsNone(self.strategy.get_instance_load(lost_instance))
 
     def test_normalize_hosts_load(self):
         self.m_model.return_value = self.fake_cluster.generate_scenario_1()
@@ -145,6 +152,12 @@ class TestWorkloadStabilization(base.TestCase):
         self.assertEqual(self.strategy.get_hosts_load(),
                          self.hosts_load_assert)
 
+    def test_get_hosts_load_with_node_missing(self):
+        self.m_model.return_value = \
+            self.fake_cluster.generate_scenario_1_with_1_node_unavailable()
+        self.assertEqual(self.hosts_load_assert,
+                         self.strategy.get_hosts_load())
+
     def test_get_sd(self):
         test_cpu_sd = 0.296
         test_ram_sd = 9.3
@@ -177,9 +190,18 @@ class TestWorkloadStabilization(base.TestCase):
     def test_simulate_migrations(self):
         model = self.fake_cluster.generate_scenario_1()
         self.m_model.return_value = model
-        self.strategy.host_choice = 'retry'
+        self.strategy.host_choice = 'fullsearch'
         self.assertEqual(
-            8,
+            10,
+            len(self.strategy.simulate_migrations(self.hosts_load_assert)))
+
+    def test_simulate_migrations_with_all_instances_exclude(self):
+        model = \
+            self.fake_cluster.generate_scenario_1_with_all_instances_exclude()
+        self.m_model.return_value = model
+        self.strategy.host_choice = 'fullsearch'
+        self.assertEqual(
+            0,
             len(self.strategy.simulate_migrations(self.hosts_load_assert)))
 
     def test_check_threshold(self):
